"""
Temperature related calculations
"""

def temp_gradient(bottom_hole_temperature, surface_temperature, bottom_hole_depth):
    """
    Temperature gradient calculation.
    Parameters
    ----------
    bottom_hole_temperature : float
        Bottom hole temperature (deg F or deg C)
    surface_temperature : float
        Surface temperature (deg F or deg C)
    bottom_hole_depth : float
        Bottom hole depth (ft or m)
    Returns
    -------
    float
        Returns temperature gradient in deg per depth unit (degF/ft or deg C/m)
    """
    gradient = (bottom_hole_temperature - surface_temperature) / bottom_hole_depth
    return gradient

def formation_temperature(surface_temperature, gradient, depth):
    """
    Calculates formation temperature based on a gradient.
    Parameters
    ----------
    surface_temperature : float
        Surface temperature (deg F or deg C)
    gradient : float
        Temperature gradient (degF/ft or degC/m)
    depth : float
        Depth at which temperature is required (ft or m)
    Returns
    -------
    float
        Returns formation temperature at a entered depth
    """
    form_temp = surface_temperature + gradient * depth
    return form_temp

print(formation_temperature(60, 0.01125, 8000))



#https://stackoverflow.com/questions/1732438/how-do-i-run-all-python-unit-tests-in-a-directory
import unittest

loader = unittest.TestLoader()

start_dir = 'tests/'

suite = loader.discover(start_dir)

result = unittest.TextTestRunner().run(suite)

if result.wasSuccessful():
    exit(0)
else:
    exit(1)



"""
unit conversion functions
"""

# Depth and length conversions
def ft_to_m(inputvalue):
    """
    Converts feet to metres.
    Parameters
    ----------
    inputvalue : float
        Input value in feet.
    Returns
    -------
    float
        Returns value in metres.
    """
    return inputvalue * 0.3048

def m_to_ft(inputvalue):
    """
    Converts metres to feet.
    Parameters
    ----------
    inputvalue : float
        Input value in metres.
    Returns
    -------
    float
        Returns value in feet.
    """
    return inputvalue * 3.28084

def ft_to_in(inputvalue):
    """
    Converts feet to inches.
    Parameters
    ----------
    inputvalue : float
        Input value in feet.
    Returns
    -------
    float
        Returns value in inches.
    """
    return inputvalue * 12

def in_to_ft(inputvalue):
    """
    Converts inches to feet.
    Parameters
    ----------
    inputvalue : float
        Input value in inches.
    Returns
    -------
    float
        Returns value in feet.
    """
    return inputvalue / 12

# Velocity / Slowness conversions
def velocity_to_slowness(inputvalue):
    """
    Converts velocity to slowness.
    Parameters
    ----------
    inputvalue : float
        Input value in velocity units.
    Returns
    -------
    float
        Returns value in slowness units.
    """
    return 1000000/inputvalue

def slowness_to_velocity(inputvalue):
    """
    Converts slowness to velocity.
    Parameters
    ----------
    inputvalue : float
        Input value in slowness units.
    Returns
    -------
    float
        Returns value in velocity units.
    """
    return 1000000/inputvalue

# Temperature conversions
def temperature_convert(inputvalue, inputunits, outputunits):
    """
    Converts temperature from one unit to another.
    Parameters
    ----------
    inputvalue : float
        Input temperature value
    inputunits : string
        Input temperature units:
            c = celsius
            f = fahrenheit
            k = kelvin
    outputunits : string
        Output temperature units:
            c = celsius
            f = fahrenheit
            k = kelvin
    Returns
    -------
    float
        Returns temperature value in required units.
    """
    if inputunits.lower() == "c":
        if outputunits.lower() == "f":
            return (9/5)*inputvalue+32
        elif outputunits.lower() == "k":
            return inputvalue + 273.15
        elif outputunits.lower() == "c":
            return inputvalue
    
    if inputunits.lower() == "f":
        if outputunits.lower() == "c":
            return (5/9)*inputvalue-32
        elif outputunits.lower() == "k":
            return (inputvalue-32) * (5/9)+273.15
        elif outputunits.lower() == "f":
            return inputvalue
    
    if inputunits.lower() == "k":
        if outputunits.lower() == "c":
            return inputvalue - 273.15
        elif outputunits.lower() == "f":
            return (inputvalue-273.15)*(9/5)+32
        elif outputunits.lower() == "k":
            return inputvalue
    units = ["k", "c", "f"]
    
    if inputunits.lower() not in units or outputunits.lower() not in units:
        raise Exception("Enter a valid temperature inputunit or outputunit value. Must be: c, f or k")



def gr_clay_shale_vol(minvalue, maxvalue, inputvalue, method="linear", limit_result=False, low_limit=0, high_limit=1):
    """
    Calculates clay or shale volume from gamma ray log.

    Maxvalue can be set to represent 100% shale or clay value when working with either VShale or VClay.
    Non-linear equations can be selected using the method argument:
        larionov-young
        larionov-old
        steiber
        clavier
    Parameters
    ----------
    minvalue : float
        Value representing a 100% clean interval.
    maxvalue : float
        Value representing either 100% clay or 100% shale.
    inputvalue : float
        Gamma ray value from log measurements.
    method : string
        Select method for calculating VClay or VShale:
        - linear
        - larionov-young
        - larionov-old
        - steiber
        - clavier

            By default: linear
    limit_result : bool, optional
        Apply limits to the result value.
        By default False
    low_limit : int, optional
        Low limit. If value falls below this limit it will be set to this value. 
        By default 0
    high_limit : float, optional
        High limit. If value falls above this limit it will be set to this value.
        By default: 1
    Returns
    -------
    float
        Returns a VShale or VClay in decimal units.

    References
    -------
    Bhuyan, K. and Passey, Q. R. (1994) Clay estimation from GR and neutron-density porosity logs, SPWLA 35th Annual Logging Symposium, pp. 1–15.
    Larionov VV (1969) Borehole radiometry: Moscow, U.S.S.R., Nedra
    Steiber RG (1973) Optimization of shale volumes in open hole logs. J Petrol Technol 1973(31):147 – 162
    Asquith and Krygowski (2004) Basic Well Log Analysis. Second Edition.
    Original paper reference cannot be found for Clavier (1971), however, the following discussion provides insight to its origins.
    https://www.researchgate.net/post/Volume_fraction_of_shale_full_reference_for_Clavier_1971
    """

    igr = (inputvalue - minvalue)/(maxvalue - minvalue)

    if method == "linear":
        result = igr
    elif method == "larionov-young":
        result = 0.083 * ((2**(3.71 * igr)) - 1)
    elif method == "larionov-old":
        result = 0.33 * ((2**(2 * igr)) - 1)
    elif method == "steiber":
        result = igr / (3 - 2 * igr)
    elif method == "clavier":
        result = 1.7 - ((3.38-(igr + 0.7)**2)**0.5)
    else:
        raise Exception(
            "Enter a valid method value: linear, larionov-young, larionov-old, steiber, clavier")

    if limit_result is True:
        return miscfuncs.limit_vals(result, low_limit, high_limit)
    else:
        return result


def sp_clay_shale_vol(minvalue, maxvalue, inputvalue, limit_result=False, low_limit=0, high_limit=1):
    """
    Calculates a clay or shale volume from SP log.

    Maxvalue can be set to represent 100% shale value when working with VShale, or it can
    be set to 100% clay when working with VClay.
    Parameters
    ----------
     minvalue : float
        Value representing a 100% clean interval.
    maxvalue : float
        Value representing either 100% clay or 100% shale.
    inputvalue : float
        Gamma ray value from log measurements.
    limit_result : bool, optional
        Apply limits to the result value.
        By default False
    low_limit : int, optional
        Low limit. If value falls below this limit it will be set to this value. 
        By default 0
    high_limit : float, optional
        High limit. If value falls above this limit it will be set to this value.
        By default: 1
    Returns
    -------
    float
        Returns a VShale or VClay in decimal units.

    References
    ----------
    Dewan, J. T., 1983, Essentials of modern open- hole log interpretation: PennWell Books, Tulsa, Oklahoma.
    """

    result = (inputvalue - minvalue)/(maxvalue - minvalue)

    if limit_result is True:
        return miscfuncs.limit_vals(result, low_limit, high_limit)
    else:
        return result


def den_neu_shale_vol(neut_porosity, dens_porosity, neut_shale_porosity, dens_shale_porosity, limit_result=False, low_limit=0, high_limit=1):
    """
    Calculates a shale volume from density and neutron porosity logs.
    Parameters
    ----------
    neut_porosity : float
        Neutron porosity
    dens_porosity : float
        Density porosity
    neut_shale_porosity : [type]
        [description]
    dens_shale_porosity : [type]
        [description]
   limit_result : bool, optional
        Apply limits to the result value.
        By default False
    low_limit : int, optional
        Low limit. If value falls below this limit it will be set to this value. 
        By default 0
    high_limit : float, optional
        High limit. If value falls above this limit it will be set to this value.
        By default: 1
    Returns
    -------
    float
        Returns a shale volume in decimal units.
    References
    ----------
    Bhuyan, K. and Passey, Q. R. (1994) Clay estimation from GR and neutron-density porosity logs, SPWLA 35th Annual Logging Symposium, pp. 1–15.
    Dewan, J. T., 1983, Essentials of modern open- hole log interpretation: PennWell Books, Tulsa, Oklahoma.
    """
    result = (neut_porosity - dens_porosity) / \
        (neut_shale_porosity - dens_shale_porosity)

    if limit_result is True:
        return miscfuncs.limit_vals(result, low_limit, high_limit)
    else:
        return result


def vshale_to_vclay(vshale, multiplier):
    """
    Converts a shale volume to clay volume using a multiplier.
    Parameters
    ----------
    vshale : float
        Shale volume
    multiplier : float
        Shale to clay multiplier (decimal)
    Returns
    -------
    float
        Returns a clay volume.
     References
    ----------
    Bhuyan, K. and Passey, Q. R. (1994) Clay estimation from GR and neutron-density porosity logs, SPWLA 35th Annual Logging Symposium, pp. 1–15.
    """
    return vshale * multiplier



from setuptools import setup, find_packages


classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Education',
    'Operating System :: MacOS',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]


setup(
    name='pyvshale',
    url='https://github.com/isaacabrahamodeh/Pyvshale',
    author='Isaac Abraham Odeh',
    author_email='isaacabrahamodeh@gmail.com',
    packages=find_packages(),
    version='0.0.1',
    description='A library of shale or clay volume calculations Gamma Ray log, SP log and, Neutro-Density log',
    long_description=open('README.txt').read() + '\n\n' +
    open('CHANGELOG.txt').read(),
    license='MIT',
    classifiers=classifiers,
    keywords='clay volume',
    install_requires=['']

)
